# SPDX-FileCopyrightText: StorPool <support@storpool.com>
# SPDX-License-Identifier: BSD-2-Clause
"""Test the workdir creation and the applying of overlays."""

from __future__ import annotations

import os
import pathlib
import re
import subprocess
from typing import TYPE_CHECKING, NamedTuple

import pytest
import utf8_locale

from sp_pkg import checkout as pcheckout
from sp_pkg import defs as pdefs


if TYPE_CHECKING:
    from typing import Final


RE_DEBCHANGELOG: Final = re.compile(
    r""" ^
    repopush \s+
    [(]
        (?P<upver> [^)-]+ )
        -
        (?P<debver> [^)]+ )
    [)]
    \s+ unstable;
    """,
    re.X,
)


class Config(NamedTuple):
    """Runtime configuration for the workdir tests."""

    base: pathlib.Path
    repopush: pathlib.Path
    repopush_version: str
    dist_deb: list[pathlib.Path]
    dist_rpm: list[pathlib.Path]


def get_env_config() -> Config | None:
    """Set up some paths based on environment variables."""
    hello_path: Final = os.environ.get("SP_HELLO")
    if not hello_path:
        print("No SP_HELLO in the environment")
        return None
    hello: Final = pathlib.Path(hello_path)

    base: Final = hello.parent.parent.parent.parent
    assert base != hello.parent.parent.parent

    repopush: Final = base / "repopush"
    debchangelog: Final = repopush / "debian/changelog"
    first_line: Final = debchangelog.read_text(encoding="UTF-8").splitlines()[0]

    fmatch: Final = RE_DEBCHANGELOG.match(first_line)
    assert fmatch
    repopush_version: Final = fmatch.group("upver")

    distdir: Final = base.parent / "dist"
    pfx_deb: Final = f"repopush_{repopush_version}.orig.tar."
    dist_deb: Final = [
        path for path in distdir.iterdir() if path.is_file() and path.name.startswith(pfx_deb)
    ]
    assert dist_deb

    pfx_rpm: Final = f"repopush-{repopush_version}.tar."
    dist_rpm: Final = [
        path for path in distdir.iterdir() if path.is_file() and path.name.startswith(pfx_rpm)
    ]
    assert dist_rpm

    return Config(
        base=base,
        repopush=repopush,
        repopush_version=repopush_version,
        dist_deb=dist_deb,
        dist_rpm=dist_rpm,
    )


@pytest.mark.parametrize(
    ("branch", "f_expected", "f_unexpected"),
    [
        ("debian/unstable", [], ["debian/compat"]),
        ("debian/bullseye", [], ["debian/compat"]),
        ("debian/stretch", [], ["debian/compat"]),
        ("ubuntu/bionic", [], ["debian/compat"]),
    ],
)
def test_workdir_simple_debian(branch: str, f_expected: list[str], f_unexpected: list[str]) -> None:
    """Test checking a Debian package out with or without overlays."""
    print()
    cfg: Final = get_env_config()
    if cfg is None:
        print("Skipping the Debian/Ubuntu workdir test")
        return

    subenv: Final = utf8_locale.get_utf8_env()

    workdir: Final = cfg.base / "work" / branch / "repopush/repopush"
    subprocess.check_call(["rm", "-rf", "--", workdir], env=subenv)

    bcfg: Final = pdefs.Config(
        branch=pdefs.BRANCHES[branch],
        branch_only=False,
        mirror=None,
        path=pdefs.Paths(
            project=cfg.base,
            package=cfg.base / "repopush",
            work=workdir,
            dist=cfg.dist_deb[0].parent,
            repo=None,
            sprepo=None,
            remote_sppkg=None,
        ),
        pkgname="repopush",
        subenv=subenv,
        verbose=True,
    )
    pcheckout.cmd_checkout(bcfg)
    assert workdir.is_dir()
    assert all((workdir.parent / path.name).is_file() for path in cfg.dist_deb)
    assert all(not (workdir.parent / path.name).exists() for path in cfg.dist_rpm)
    for name in ["README.md", "repopush.scd", "debian/control", *f_expected]:
        assert (workdir / name).is_file()
    for name in f_unexpected:
        assert not (workdir / name).exists()


@pytest.mark.parametrize("branch", ["centos/7", "centos/8"])
def test_workdir_centos(branch: str) -> None:
    """Test checking a CentOS package out without overlays."""
    print()
    cfg: Final = get_env_config()
    if cfg is None:
        print("Skipping the CentOS workdir test")
        return

    subenv: Final = utf8_locale.get_utf8_env()

    workdir: Final = cfg.base / "work" / branch / "repopush"
    subprocess.check_call(["rm", "-rf", "--", workdir], env=subenv)

    bcfg: Final = pdefs.Config(
        branch=pdefs.BRANCHES[branch],
        branch_only=False,
        mirror=None,
        path=pdefs.Paths(
            project=cfg.base,
            package=cfg.base / "repopush",
            work=cfg.base / "work" / branch / "repopush",
            dist=cfg.dist_rpm[0].parent,
            repo=None,
            sprepo=None,
            remote_sppkg=None,
        ),
        pkgname="repopush",
        subenv=subenv,
        verbose=True,
    )
    pcheckout.cmd_checkout(bcfg)
    srcdir: Final = workdir / "src"
    assert srcdir.is_dir()
    assert (srcdir / "repopush.spec").is_file()
    assert all((srcdir / path.name).is_file() for path in cfg.dist_rpm)
    assert all(not (srcdir / path.name).exists() for path in cfg.dist_deb)
    assert all(not (workdir.parent / path.name).exists() for path in cfg.dist_deb + cfg.dist_rpm)
